@@ -2,8 +2,6 @@
 
 #include <config.h>
 
-#include <pthread.h>
-
 #include <string.h>
 
 #include "e-book.h"
@@ -27,6 +25,7 @@ GMainContext *_ebook_context;
 
 static GObjectClass *parent_class;
 
+#ifdef __PRETTY_FUNCTION__
 #define e_return_error_if_fail(expr,error_code)	G_STMT_START{		\
      if G_LIKELY(expr) { } else						\
        {								\
@@ -45,6 +44,24 @@ static GObjectClass *parent_class;
 		#expr);							\
 	 return FALSE;							\
        };				}G_STMT_END
+#else
+#define e_return_error_if_fail(expr,error_code)	G_STMT_START{		\
+     if G_LIKELY(expr) { } else						\
+       {								\
+	 g_log (G_LOG_DOMAIN,						\
+		G_LOG_LEVEL_CRITICAL,					\
+		"file %s: line %d: assertion `%s' failed",		\
+		__FILE__,						\
+		__LINE__,						\
+		#expr);							\
+	 g_set_error (error, E_BOOK_ERROR, (error_code),                \
+		"file %s: line %d: assertion `%s' failed",		\
+		__FILE__,						\
+		__LINE__,						\
+		#expr);							\
+	 return FALSE;							\
+       };				}G_STMT_END
+#endif
 
 /* XXX we need a better error message here */
 #define E_BOOK_CHECK_STATUS(status,error) G_STMT_START{			\
@@ -3720,7 +3737,7 @@ e_book_activate()
 
 	g_static_mutex_lock (&e_book_lock);
 	if (!activated) {
-		pthread_t ebook_mainloop_thread;
+		GThread *ebook_mainloop_thread;
 		activated = TRUE;
 
 		_ebook_context = g_main_context_new ();
@@ -3728,7 +3745,7 @@ e_book_activate()
 		if (!bonobo_is_initialized ())
 			bonobo_init (NULL, NULL);
 
-		pthread_create(&ebook_mainloop_thread, NULL, startup_mainloop, NULL);
+		ebook_mainloop_thread = g_thread_create(startup_mainloop, NULL, FALSE, NULL);
 	}
 	g_static_mutex_unlock (&e_book_lock);
 }
