@@ -343,6 +343,7 @@ static void
 set_attendees_to_item (EGwItem *item, ECalComponent *comp, icaltimezone *default_zone, gboolean delegate, const char *user_email)
 {
 	if (e_cal_component_has_attendees (comp)) {
+		gboolean att_changed = FALSE;
 		GSList *attendee_list, *recipient_list = NULL, *al;
 
 		e_cal_component_get_attendee_list (comp, &attendee_list);	
@@ -353,6 +354,11 @@ set_attendees_to_item (EGwItem *item, ECalComponent *comp, icaltimezone *default
 			if (delegate && (g_str_equal (attendee->value + 7, user_email) || !(attendee->delfrom && *attendee->delfrom)))
 				continue;		
 			
+			if (delegate) {
+				attendee->delfrom = "";
+				att_changed = TRUE;
+			}
+	
 			recipient = g_new0 (EGwItemRecipient, 1);
 
 			/* len (MAILTO:) + 1 = 7 */
@@ -375,6 +381,13 @@ set_attendees_to_item (EGwItem *item, ECalComponent *comp, icaltimezone *default
 			recipient_list = g_slist_append (recipient_list, recipient);
 		}
 
+		/* Reset the ECalComponent with changes if necessary*/
+		if(att_changed)
+			e_cal_component_set_attendee_list (comp, attendee_list);	
+
+		e_cal_component_free_attendee_list(attendee_list);
+	
+		/* recipient_list shouldn't be freed. Look into the function below. */
 		e_gw_item_set_recipient_list (item, recipient_list);
 
 		/* Send Options */
@@ -928,21 +941,23 @@ e_gw_item_to_cal_component (EGwItem *item, ECalBackendGroupwise *cbgw)
 	t = e_gw_item_get_start_date (item);
 	if (t) {
 		itt_utc = icaltime_from_string (t);
-		if (!icaltime_get_timezone (itt_utc))
-			icaltime_set_timezone (&itt_utc, icaltimezone_get_utc_timezone());
-		if (default_zone) {
-			itt = icaltime_convert_to_zone (itt_utc, default_zone); 
-			icaltime_set_timezone (&itt, default_zone);
-			dt.value = &itt;
-			dt.tzid = icaltimezone_get_tzid (default_zone);
+		
+		if (!is_allday) {
+			if (!icaltime_get_timezone (itt_utc))
+				icaltime_set_timezone (&itt_utc, icaltimezone_get_utc_timezone());
+			if (default_zone) {
+				itt = icaltime_convert_to_zone (itt_utc, default_zone); 
+				icaltime_set_timezone (&itt, default_zone);
+				dt.value = &itt;
+				dt.tzid = icaltimezone_get_tzid (default_zone);
+			} else {
+				dt.value = &itt_utc;
+				dt.tzid = g_strdup ("UTC");
+			}
 		} else {
 			dt.value = &itt_utc;
-			dt.tzid = g_strdup ("UTC");
-		}
-		if (is_allday) {
-			dt.value->is_date = 1;
-			dt.tzid = NULL;
-		}
+		}	
+
 		e_cal_component_set_dtstart (comp, &dt);
 		g_free (t);
 	}
@@ -1054,24 +1069,26 @@ e_gw_item_to_cal_component (EGwItem *item, ECalBackendGroupwise *cbgw)
 		t = e_gw_item_get_end_date (item);
 		if (t) {
 			itt_utc = icaltime_from_string (t);
-			if (!icaltime_get_timezone (itt_utc))
-				icaltime_set_timezone (&itt_utc, icaltimezone_get_utc_timezone());
-			if (default_zone) {
-				itt = icaltime_convert_to_zone (itt_utc, default_zone); 
-				icaltime_set_timezone (&itt, default_zone);
-				dt.value = &itt;
-				dt.tzid = icaltimezone_get_tzid (default_zone);
+
+			if (!is_allday) {
+				if (!icaltime_get_timezone (itt_utc))
+					icaltime_set_timezone (&itt_utc, icaltimezone_get_utc_timezone());
+				if (default_zone) {
+					itt = icaltime_convert_to_zone (itt_utc, default_zone); 
+					icaltime_set_timezone (&itt, default_zone);
+					dt.value = &itt;
+					dt.tzid = icaltimezone_get_tzid (default_zone);
+				} else {
+					dt.value = &itt_utc;
+					dt.tzid = g_strdup ("UTC");
+				}
 			} else {
 				dt.value = &itt_utc;
-				dt.tzid = g_strdup ("UTC");
 			}
-			if (is_allday) {
-				dt.value->is_date = 1;
-				dt.tzid = NULL;
-			}	
+
 			e_cal_component_set_dtend (comp, &dt);
 		}
-		
+
 
 		/* alarms*/
 		/* we negate the value as GW supports only "before" the start of event alarms */
@@ -1083,7 +1100,7 @@ e_gw_item_to_cal_component (EGwItem *item, ECalBackendGroupwise *cbgw)
 			alarm = e_cal_component_alarm_new ();
 			e_cal_component_alarm_set_action (alarm, E_CAL_COMPONENT_ALARM_DISPLAY);
 			trigger.type = E_CAL_COMPONENT_ALARM_TRIGGER_RELATIVE_START;
-			trigger.u.rel_duration = (struct icaldurationtype) icaldurationtype_from_int (alarm_duration);
+			trigger.u.rel_duration = icaldurationtype_from_int (alarm_duration);
 			e_cal_component_alarm_set_trigger (alarm, trigger);
 			e_cal_component_add_alarm (comp, alarm);
 		}
@@ -1165,7 +1182,8 @@ e_gw_connection_send_appointment (ECalBackendGroupwise *cbgw, const char *contai
 	EGwConnection *cnc;
 	EGwConnectionStatus status;
 	icalparameter_partstat partstat;
-	char *item_id;
+	char *item_id = NULL;
+	const char *gw_id;
 	gboolean all_instances = FALSE;
 	icalproperty *icalprop;
 	icalcomponent *icalcomp;
@@ -1188,31 +1206,38 @@ e_gw_connection_send_appointment (ECalBackendGroupwise *cbgw, const char *contai
 		const char *x_name;
 
 		x_name = icalproperty_get_x_name (icalprop);
-		if (!strcmp (x_name, "X-GW-RECUR-INSTANCE-MOD-TYPE")) {
+		if (!strcmp (x_name, "X-GW-RECUR-INSTANCES-MOD-TYPE")) {
 			if (!strcmp (icalproperty_get_x (icalprop), "All"))
 				all_instances = TRUE;
 			if (recurrence_key)
 				break;
 		}
 		if (!strcmp (x_name, "X-GW-RECURRENCE-KEY")) {
-			recurrence_key = icalproperty_get_x (icalprop);
+			e_cal_component_get_uid (comp, &recurrence_key);
 		}
 		icalprop = icalcomponent_get_next_property (icalcomp, ICAL_X_PROPERTY);
 	}	
 		
-	if (all_instances) {
-		switch  (e_cal_component_get_vtype (comp)) {
+	gw_id = e_cal_component_get_gw_id (comp);
+
+	switch  (e_cal_component_get_vtype (comp)) {
+
 		case E_CAL_COMPONENT_EVENT: 
-			item_id = g_strconcat (e_cal_component_get_gw_id (comp), GW_EVENT_TYPE_ID, container, NULL);
+			if (!g_str_has_suffix (gw_id, container))
+				item_id = g_strconcat (e_cal_component_get_gw_id (comp), GW_EVENT_TYPE_ID, container, NULL);
+			else 
+				item_id = g_strdup (gw_id);
 			break;
 		case E_CAL_COMPONENT_TODO:
-			item_id = g_strconcat (e_cal_component_get_gw_id (comp), GW_TODO_TYPE_ID, container, NULL);
+			if (!g_str_has_suffix (gw_id, container))
+				item_id = g_strconcat (e_cal_component_get_gw_id (comp), GW_TODO_TYPE_ID, container, NULL);
+			else
+				item_id = g_strdup (gw_id);
 			break;
 		default:
 			return E_GW_CONNECTION_STATUS_INVALID_OBJECT;
-		}
 	}
-	
+
 	switch (method) {
 	case ICAL_METHOD_REQUEST:
 		/* get attendee here and add the list along. */
@@ -1257,29 +1282,30 @@ e_gw_connection_send_appointment (ECalBackendGroupwise *cbgw, const char *contai
 			e_cal_component_get_transparency (comp, &transp);
 			if (transp == E_CAL_COMPONENT_TRANSP_OPAQUE)  {
 				if (all_instances)
-					status = e_gw_connection_accept_request_by_recurrence_key (cnc, recurrence_key, "Busy", NULL);
+					status = e_gw_connection_accept_request (cnc, item_id, "Busy",  NULL, recurrence_key);
 				else
-					status = e_gw_connection_accept_request (cnc, item_id, "Busy");
+					status = e_gw_connection_accept_request (cnc, item_id, "Busy", NULL, NULL);
 			}
 			else {
 				if (all_instances)
-					status = e_gw_connection_accept_request_by_recurrence_key (cnc, recurrence_key, "Free", NULL);
+					status = e_gw_connection_accept_request (cnc, item_id, "Free",  NULL, recurrence_key);
 				else
-					status = e_gw_connection_accept_request (cnc, item_id, "Free");
+					status = e_gw_connection_accept_request (cnc, item_id, "Free", NULL, NULL);
 			}
 			break;
 		case ICAL_PARTSTAT_DECLINED:
 			if (all_instances)
-				status = e_gw_connection_decline_request_by_recurrence_key (cnc, recurrence_key, NULL);
+				status = e_gw_connection_decline_request (cnc, item_id, NULL, recurrence_key);
 			else
-				status = e_gw_connection_decline_request (cnc, item_id);
+				status = e_gw_connection_decline_request (cnc, item_id, NULL, NULL);
+			
 			*remove = TRUE;
 			break;
 		case ICAL_PARTSTAT_TENTATIVE:
 			if (all_instances)
-				status = e_gw_connection_accept_request_by_recurrence_key (cnc, recurrence_key, "Tentative", NULL);
+				status = e_gw_connection_accept_request (cnc, item_id, "Tentative", NULL, recurrence_key);
 			else
-				status = e_gw_connection_accept_request (cnc, item_id, "Tentative");
+				status = e_gw_connection_accept_request (cnc, item_id, "Tentative", NULL, NULL);
 			break;
 		case ICAL_PARTSTAT_COMPLETED:
 			status = e_gw_connection_complete_request (cnc, item_id);
@@ -1465,14 +1491,14 @@ e_gw_connection_get_freebusy_info (EGwConnection *cnc, GList *users, time_t star
         SoupSoapMessage *msg;
         SoupSoapResponse *response;
         EGwConnectionStatus status;
-        SoupSoapParameter *param, *subparam;
-        const char *session;
+        SoupSoapParameter *param, *subparam, *param_outstanding;
+        const char *session, *outstanding = NULL;
 	gboolean resend_request = TRUE;
 	int request_iteration = 0;
 
 	g_return_val_if_fail (E_IS_GW_CONNECTION (cnc), E_GW_CONNECTION_STATUS_INVALID_CONNECTION);
 
-        /* Perform startFreeBusySession */
+	/* Perform startFreeBusySession */
         status = start_freebusy_session (cnc, users, start, end, &session); 
         /*FIXME log error messages  */
         if (status != E_GW_CONNECTION_STATUS_OK)
@@ -1503,6 +1529,26 @@ e_gw_connection_get_freebusy_info (EGwConnection *cnc, GList *users, time_t star
                 return status;
         }
 
+	param = soup_soap_response_get_first_parameter_by_name (response, "freeBusyStats");
+        if (!param) {
+                g_object_unref (response);
+                g_object_unref (msg);
+                return E_GW_CONNECTION_STATUS_INVALID_RESPONSE;
+        }
+
+	param_outstanding = soup_soap_parameter_get_first_child_by_name (param, "outstanding");
+	if (param_outstanding)
+		outstanding = soup_soap_parameter_get_string_value (param_outstanding);
+	/* Try 12 times - this is approximately 2 minutes of time to
+	 * obtain the free/busy information from the server */
+	if (outstanding && strcmp (outstanding, "0") && (request_iteration < 12)) {
+		request_iteration++;
+		g_object_unref (msg);
+		g_object_unref (response);
+		g_usleep (10000000);
+		goto resend;
+	}
+
         /* FIXME  the FreeBusyStats are not used currently.  */
         param = soup_soap_response_get_first_parameter_by_name (response, "freeBusyInfo");
         if (!param) {
@@ -1511,27 +1557,6 @@ e_gw_connection_get_freebusy_info (EGwConnection *cnc, GList *users, time_t star
                 return E_GW_CONNECTION_STATUS_INVALID_RESPONSE;
         }
 
-	for (subparam = soup_soap_parameter_get_first_child_by_name (param, "freeBusyStats");
-	     subparam != NULL;
-	     subparam = soup_soap_parameter_get_next_child_by_name (subparam, "freeBusyStats")) {
-		SoupSoapParameter *param_outstanding;
-		const char *outstanding = NULL;
-
-		param_outstanding = soup_soap_parameter_get_first_child_by_name (subparam, "outstanding");
-		if (param_outstanding)
-			outstanding = soup_soap_parameter_get_string_value (param_outstanding);
-		/* Try 12 times - this is approximately 2 minutes of time to
-		 * obtain the free/busy information from the server */
-		if (outstanding && strcmp (outstanding, "0") && (request_iteration < 12)) {
-			request_iteration++;
-			g_object_unref (msg);
-		        g_object_unref (response);
-			g_usleep (10000000);
-			goto resend;
-		}
-
-	}
-
 	resend_request = FALSE;
 
         for (subparam = soup_soap_parameter_get_first_child_by_name (param, "user");
@@ -1875,22 +1900,27 @@ e_cal_backend_groupwise_store_settings (EGwSendOptions *opts, ECalBackendGroupwi
 gboolean
 e_cal_backend_groupwise_utils_check_delegate (ECalComponent *comp, const char *email)
 {
-	GSList *attendee_list, *l;
-	ECalComponentAttendee  *attendee = NULL;
+	icalproperty *prop;	
+	icalcomponent *icalcomp = e_cal_component_get_icalcomponent (comp);
+	
+	/*TODO remove the argument email */
+	prop = icalcomponent_get_first_property (icalcomp,
+						 ICAL_X_PROPERTY);
+	while (prop) {
+		const char *x_name, *x_val;
 
-
-	e_cal_component_get_attendee_list (comp, &attendee_list);
-	for (l = attendee_list; l ; l = g_slist_next (l)) {
-		attendee = l->data;
-
-		if (g_str_equal (attendee->value + 7, email)) {
-		       	if (attendee->status == ICAL_PARTSTAT_DELEGATED)
-				return TRUE;
-			else
-				return FALSE;
+		x_name = icalproperty_get_x_name (prop);
+		x_val = icalproperty_get_x (prop);
+		if (!strcmp (x_name, "X-EVOLUTION-DELEGATED")) {
+			icalcomponent_remove_property (icalcomp, prop);
+			return TRUE;
 		}
-	}
 
-	return FALSE;
+		prop = icalcomponent_get_next_property (e_cal_component_get_icalcomponent (comp),
+							ICAL_X_PROPERTY);
+ 	}
+ 
+ 	return FALSE;
+
 }
 
