@@ -467,9 +467,14 @@ update_junk_list (CamelStore *store, CamelMessageInfo *info, int flag)
 	CamelGroupwiseStorePrivate  *priv = gw_store->priv;
 	EGwConnection *cnc = cnc_lookup (priv);
 
-	from = g_strdup (camel_message_info_from (info));
+	if (!(from = g_strdup (camel_message_info_from (info))))
+		goto error;
+
 	email = g_strsplit_set (from, "<>", -1);
-	
+
+	if (!email[1])
+		goto error;
+
 	if (e_gw_connection_get_junk_entries (cnc, &list)== E_GW_CONNECTION_STATUS_OK){
 		while (list) {
 			entry = list->data;
@@ -487,6 +492,7 @@ update_junk_list (CamelStore *store, CamelMessageInfo *info, int flag)
 			if (e_gw_connection_create_junk_entry (cnc, email[1], "email", "junk") == E_GW_CONNECTION_STATUS_OK);
 		g_list_foreach (list, (GFunc) free_node, NULL);
 	}
+error:	
 	g_free (from);
 	g_strfreev (email);
 }
@@ -600,7 +606,7 @@ groupwise_sync (CamelFolder *folder, gboolean expunge, CamelException *ex)
 			}
 		}
 		camel_message_info_free (info);
-		
+		info = NULL;
 	}
 
 	if (read_items && g_list_length (read_items)) {
@@ -746,7 +752,13 @@ static CamelSessionThreadOps update_ops = {
 static void
 groupwise_refresh_info(CamelFolder *folder, CamelException *ex)
 {
-	groupwise_refresh_folder(folder, ex);
+	CamelGroupwiseSummary *summary;
+	summary = (CamelGroupwiseSummary *) folder->summary;
+	if (summary->time_string) {
+		g_print ("Refreshing folder:%s\n", summary->time_string);
+		groupwise_refresh_folder(folder, ex);
+	} else 
+		g_print ("Not refreshing folder\n");
 }
 
 void
@@ -794,15 +806,20 @@ groupwise_refresh_folder(CamelFolder *folder, CamelException *ex)
 		g_print ("get_items:%s\n", folder->full_name);
 		status = e_gw_connection_get_items (cnc, container_id, "recipient distribution created attachments subject status size", NULL, &list);
 		if (status != E_GW_CONNECTION_STATUS_OK) {
-			camel_exception_set (ex, CAMEL_EXCEPTION_SERVICE_INVALID, _("Authentication failed"));
+			if (status ==E_GW_CONNECTION_STATUS_OTHER) {
+				g_warning ("Trash full....Empty Trash!!!!\n");
+				/*groupwise_expunge (folder, ex);*/
+			} else
+				camel_exception_set (ex, CAMEL_EXCEPTION_SERVICE_INVALID, _("Authentication failed"));
 			goto end1;
 		}
-		if (!list || !g_list_length(list))
+		if (list || g_list_length(list)) {
 			camel_folder_summary_clear (folder->summary);
-		gw_update_summary (folder, list, ex);
-		g_list_foreach (list, (GFunc) g_object_unref, NULL);
-		g_list_free (list);
-		list = NULL;
+			gw_update_summary (folder, list, ex);
+			g_list_foreach (list, (GFunc) g_object_unref, NULL);
+			g_list_free (list);
+			list = NULL;
+		}
 		goto end1;
 	}
 
@@ -855,7 +872,8 @@ groupwise_refresh_folder(CamelFolder *folder, CamelException *ex)
 		gw_store->current_folder = folder;
 	}
 
-	gw_update_cache (folder, list, ex);
+	if (list)
+		gw_update_cache (folder, list, ex);
 	
 
 	/*
@@ -907,6 +925,7 @@ gw_update_cache ( CamelFolder *folder, GList *item_list,CamelException *ex)
 	char *container_id = NULL;
 	gboolean is_junk = FALSE;
 	EGwItemStatus status;
+	int total_items = g_list_length (item_list), i=0;
 	
 	/*Assert lock*/
 	msg = g_ptr_array_new ();
@@ -921,6 +940,8 @@ gw_update_cache ( CamelFolder *folder, GList *item_list,CamelException *ex)
 		is_junk = TRUE;
 	}
 
+	camel_operation_start (NULL, _("Fetching summary information for new messages"));
+
 	for ( ; item_list != NULL ; item_list = g_list_next (item_list) ) {
 		EGwItem *temp_item = (EGwItem *)item_list->data;
 		EGwItem *item;
@@ -935,16 +956,19 @@ gw_update_cache ( CamelFolder *folder, GList *item_list,CamelException *ex)
 
 		id = e_gw_item_get_id (temp_item);
 
+		camel_operation_progress (NULL, (100*i)/total_items);
 		cache_stream  = camel_data_cache_get (gw_folder->cache, "cache", id, ex);
 		if (cache_stream) {
 			camel_object_unref (cache_stream);
 			cache_stream = NULL;
+			i++;
 			//g_print ("*** Exists in cache, continuing....%s\n", id);
 			continue;
 		} 
 
 		status = e_gw_connection_get_item (cnc, container_id, id, "peek default distribution recipient message attachments subject notification created recipientStatus status", &item);
 		if (status != E_GW_CONNECTION_STATUS_OK) {
+			i++;
 			continue;
 		}
 
@@ -1071,7 +1095,9 @@ gw_update_cache ( CamelFolder *folder, GList *item_list,CamelException *ex)
 		}
 		/******************** Caching stuff ends *************************/
 		g_object_unref (item);
+		i++;
 	}
+	camel_operation_end (NULL);
 	g_free (container_id);
 	g_string_free (str, TRUE);
 	camel_object_trigger_event (folder, "folder_changed", changes);
@@ -1241,7 +1267,7 @@ groupwise_folder_item_to_msg( CamelFolder *folder,
 	CamelMimeMessage *msg = NULL;
 	CamelGroupwiseStore *gw_store = CAMEL_GROUPWISE_STORE(folder->parent_store);
 	CamelGroupwiseStorePrivate  *priv = gw_store->priv;
-	char *container_id;
+	char *container_id = NULL;
 	GSList *attach_list = NULL;
 	EGwItemType type;
 	EGwConnectionStatus status;
@@ -1254,6 +1280,7 @@ groupwise_folder_item_to_msg( CamelFolder *folder,
 
 	uid = e_gw_item_get_id(item);
 	cnc = cnc_lookup (priv);
+	container_id = camel_groupwise_store_container_id_lookup (gw_store, folder->full_name);
 
 	attach_list = e_gw_item_get_attach_id_list (item);
 	if (attach_list) {
@@ -1368,13 +1395,22 @@ groupwise_folder_item_to_msg( CamelFolder *folder,
 				}
 				if (attachment && (len !=0) ) {
 					part = camel_mime_part_new ();
+					
+					if (!strcmp (attach->contentType, "application/pgp-signature")) {
+						camel_data_wrapper_set_mime_type(CAMEL_DATA_WRAPPER (multipart), "multipart/signed");
+						camel_content_type_set_param(CAMEL_DATA_WRAPPER (multipart)->mime_type, "protocol", attach->contentType);
+					} else if (!strcmp (attach->contentType, "application/pgp-encrypted")) {
+						camel_data_wrapper_set_mime_type(CAMEL_DATA_WRAPPER (multipart), "multipart/encrypted");
+ 						camel_content_type_set_param(CAMEL_DATA_WRAPPER (multipart)->mime_type, "protocol", attach->contentType);
+					} else {
+						camel_mime_part_set_content_id (part, attach->id);
+						camel_data_wrapper_set_mime_type(CAMEL_DATA_WRAPPER (multipart), "multipart/digest");
+					}
 
-					camel_data_wrapper_set_mime_type(CAMEL_DATA_WRAPPER (multipart), "multipart/digest");
 					camel_multipart_set_boundary(multipart, NULL);
 
 					camel_mime_part_set_filename(part, g_strdup(attach->name));
 					camel_mime_part_set_content(part, attachment, len, attach->contentType);
-					camel_mime_part_set_content_id (part, attach->id);
 
 					camel_multipart_add_part (multipart, part);
 
