@@ -70,6 +70,7 @@
 #include "camel-file-utils.h"
 #include "camel-debug.h"
 #include "camel-i18n.h"
+#include "camel/camel-store-summary.h"
 
 #define d(x) 
 
@@ -496,6 +497,7 @@ imap_refresh_info (CamelFolder *folder, CamelException *ex)
 	CamelImapStore *imap_store = CAMEL_IMAP_STORE (folder->parent_store);
 	CamelImapFolder *imap_folder = CAMEL_IMAP_FOLDER (folder);
 	CamelImapResponse *response;
+	CamelStoreInfo *si;
 
 	if (camel_disco_store_status (CAMEL_DISCO_STORE (imap_store)) == CAMEL_DISCO_STORE_OFFLINE)
 		return;
@@ -511,6 +513,10 @@ imap_refresh_info (CamelFolder *folder, CamelException *ex)
 	 * us with a NOOP of new messages, so force a reselect which
 	 * should do it.  */
 	CAMEL_SERVICE_LOCK (imap_store, connect_lock);
+
+	if (!camel_imap_store_connected(imap_store, ex))
+		goto done;
+
 	if (imap_store->current_folder != folder
 	    || g_ascii_strcasecmp(folder->full_name, "INBOX") == 0) {
 		response = camel_imap_command (imap_store, folder, ex, NULL);
@@ -537,7 +543,56 @@ imap_refresh_info (CamelFolder *folder, CamelException *ex)
 		camel_imap_response_free (imap_store, response);
 	}
 
+	si = camel_store_summary_path((CamelStoreSummary *)((CamelImapStore *)folder->parent_store)->summary, folder->full_name);
+	if (si) {
+		guint32 unread, total;
+
+		camel_object_get(folder, NULL, CAMEL_FOLDER_TOTAL, &total, CAMEL_FOLDER_UNREAD, &unread, NULL);
+		if (si->total != total
+		    || si->unread != unread){
+			si->total = total;
+			si->unread = unread;
+			camel_store_summary_touch((CamelStoreSummary *)((CamelImapStore *)folder->parent_store)->summary);
+		}
+		camel_store_summary_info_free((CamelStoreSummary *)((CamelImapStore *)folder->parent_store)->summary, si);
+	}
+done:
 	CAMEL_SERVICE_UNLOCK (imap_store, connect_lock);
+
+	camel_folder_summary_save(folder->summary);
+	camel_store_summary_save((CamelStoreSummary *)((CamelImapStore *)folder->parent_store)->summary);
+}
+
+static void
+flags_to_label(CamelFolder *folder, CamelImapMessageInfo *mi)
+{
+	/* We snoop the server flag setting, and map it to
+	   the label 'user tag'.  We also clean it up at this
+	   point, there can only be 1 label set at a time */
+	if (folder->permanent_flags & CAMEL_IMAP_MESSAGE_LABEL_MASK) {
+		const char *label = NULL;
+		guint32 mask = 0;
+
+		if (mi->info.flags & CAMEL_IMAP_MESSAGE_LABEL1) {
+			mask = CAMEL_IMAP_MESSAGE_LABEL1;
+			label = "important";
+		} else if (mi->info.flags & CAMEL_IMAP_MESSAGE_LABEL2) {
+			mask = CAMEL_IMAP_MESSAGE_LABEL2;
+			label = "work";
+		} else if (mi->info.flags & CAMEL_IMAP_MESSAGE_LABEL3) {
+			mask = CAMEL_IMAP_MESSAGE_LABEL3;
+			label = "personal";
+		} else if (mi->info.flags & CAMEL_IMAP_MESSAGE_LABEL4) {
+			mask = CAMEL_IMAP_MESSAGE_LABEL4;
+			label = "todo";
+		} else if (mi->info.flags & CAMEL_IMAP_MESSAGE_LABEL5) {
+			mask = CAMEL_IMAP_MESSAGE_LABEL5;
+			label = "later";
+		}
+
+		mi->info.flags = (mi->info.flags & ~CAMEL_IMAP_MESSAGE_LABEL_MASK) | mask;
+		camel_tag_set(&mi->info.user_tags, "label", label);
+	}
 }
 
 /* Called with the store's connect_lock locked */
@@ -653,8 +708,9 @@ imap_rescan (CamelFolder *folder, int exists, CamelException *ex)
 			if (changes == NULL)
 				changes = camel_folder_change_info_new();
 			camel_folder_change_info_change_uid(changes, new[i].uid);
+			flags_to_label(folder, (CamelImapMessageInfo *)info);
 		}
-		
+
 		camel_message_info_free(info);
 		g_free (new[i].uid);
 	}
@@ -751,6 +807,7 @@ static void
 imap_sync_offline (CamelFolder *folder, CamelException *ex)
 {
 	camel_folder_summary_save (folder->summary);
+	camel_store_summary_save((CamelStoreSummary *)((CamelImapStore *)folder->parent_store)->summary);
 }
 
 static void
@@ -765,11 +822,11 @@ imap_sync_online (CamelFolder *folder, CamelException *ex)
 	gboolean unset;
 	int i, j, max;
 	
-	if (((CamelImapFolder *)folder)->read_only) {
+	if (folder->permanent_flags == 0) {
 		imap_sync_offline (folder, ex);
 		return;
 	}
-
+	
 	camel_exception_init (&local_ex);
 	CAMEL_SERVICE_LOCK (store, connect_lock);
 	
@@ -786,7 +843,7 @@ imap_sync_online (CamelFolder *folder, CamelException *ex)
 			camel_message_info_free((CamelMessageInfo *)info);
 			continue;
 		}
-		
+
 		/* Note: Cyrus is broken and will not accept an
 		   empty-set of flags so... if this is true then we
 		   want to unset the previously set flags.*/
@@ -803,6 +860,12 @@ imap_sync_online (CamelFolder *folder, CamelException *ex)
 		if (matches == NULL)
 			continue;
 		
+		/* Make sure we're connected before issuing commands */
+		if (!camel_imap_store_connected(store, ex)) {
+			g_free(set);
+			break;
+		}
+
 		/* FIXME: since we don't know the previously set flags,
 		   if unset is TRUE then just unset all the flags? */
 		flaglist = imap_create_flag_list (unset ? folder->permanent_flags : info->info.flags & folder->permanent_flags);
@@ -1167,12 +1230,6 @@ do_append (CamelFolder *folder, CamelMimeMessage *message,
 	GByteArray *ba;
 	char *flagstr, *end;
 	guint32 flags;
-
-	flags = camel_message_info_flags(info);
-	if (flags)
-		flagstr = imap_create_flag_list (flags);
-	else
-		flagstr = NULL;
 	
 	/* encode any 8bit parts so we avoid sending embedded nul-chars and such  */
 	camel_mime_message_encode_8bit_parts (message);
@@ -1191,6 +1248,18 @@ do_append (CamelFolder *folder, CamelMimeMessage *message,
 	camel_object_unref (CAMEL_OBJECT (streamfilter));
 	camel_object_unref (CAMEL_OBJECT (crlf_filter));
 	camel_object_unref (CAMEL_OBJECT (memstream));
+
+	/* Some servers dont let us append with custom flags.  If the command fails for
+	   whatever reason, assume this is the case and save the state and try again */
+retry:
+	flags = camel_message_info_flags(info);
+	if (!store->nocustomappend)
+		flags |= imap_label_to_flags((CamelMessageInfo *)info);
+	flags &= folder->permanent_flags;
+	if (flags)
+		flagstr = imap_create_flag_list (flags);
+	else
+		flagstr = NULL;
 	
 	response = camel_imap_command (store, NULL, ex, "APPEND %F%s%s {%d}",
 				       folder->full_name, flagstr ? " " : "",
@@ -1198,6 +1267,11 @@ do_append (CamelFolder *folder, CamelMimeMessage *message,
 	g_free (flagstr);
 	
 	if (!response) {
+		if (camel_exception_get_id(ex) == CAMEL_EXCEPTION_SERVICE_INVALID && !store->nocustomappend) {
+			camel_exception_clear(ex);
+			store->nocustomappend = 1;
+			goto retry;
+		}
 		g_byte_array_free (ba, TRUE);
 		return NULL;
 	}
@@ -1456,7 +1530,7 @@ do_copy (CamelFolder *source, GPtrArray *uids,
 				handle_copyuid (response, source, destination);
 			camel_imap_response_free (store, response);
 
-			if (!camel_exception_is_set(ex)) {
+			if (!camel_exception_is_set(ex) && delete_originals) {
 				for (i=last;i<uid;i++)
 					camel_folder_delete_message(source, uids->pdata[i]);
 				last = uid;
@@ -1994,11 +2068,6 @@ imap_get_message (CamelFolder *folder, const char *uid, CamelException *ex)
 		retry++;
 		camel_exception_clear(ex);
 
-		/* If we are online, make sure we're also connected */
-		if (camel_disco_store_status((CamelDiscoStore *)store) == CAMEL_DISCO_STORE_ONLINE
-		    &&  !camel_imap_store_connected(store, ex))
-			goto fail;
-	
 		/* If the message is small or only 1 part, or server doesn't do 4v1 (properly) fetch it in one piece. */
 		if (store->server_level < IMAP_LEVEL_IMAP4REV1
 		    || store->braindamaged
@@ -2017,13 +2086,17 @@ imap_get_message (CamelFolder *folder, const char *uid, CamelException *ex)
 				char *body, *found_uid;
 				int i;
 				
-				if (camel_disco_store_status (CAMEL_DISCO_STORE (store)) == CAMEL_DISCO_STORE_OFFLINE) {
+				CAMEL_SERVICE_LOCK(store, connect_lock);
+				if (!camel_imap_store_connected(store, ex)) {
+					CAMEL_SERVICE_UNLOCK(store, connect_lock);
 					camel_exception_set (ex, CAMEL_EXCEPTION_SERVICE_UNAVAILABLE,
 							     _("This message is not currently available"));
 					goto fail;
 				}
 				
 				response = camel_imap_command (store, folder, ex, "UID FETCH %s BODY", uid);
+				CAMEL_SERVICE_UNLOCK(store, connect_lock);
+
 				if (response) {
 					for (i = 0, body = NULL; i < response->untagged->len; i++) {
 						fetch_data = parse_fetch_response (imap_folder, response->untagged->pdata[i]);
@@ -2038,13 +2111,18 @@ imap_get_message (CamelFolder *folder, const char *uid, CamelException *ex)
 						}
 					}
 					
-					if (body)
+					if (body) {
+						/* NB: small race here, setting the info.content */
 						imap_parse_body ((const char **) &body, folder, mi->info.content);
-					
+						camel_folder_summary_touch (folder->summary);
+					}
+
 					if (fetch_data)
 						g_datalist_clear (&fetch_data);
 					
 					camel_imap_response_free (store, response);
+				} else {
+					camel_exception_clear(ex);
 				}
 			}
 
@@ -2261,7 +2339,7 @@ imap_update_summary (CamelFolder *folder, int exists,
 	
 	CAMEL_SERVICE_ASSERT_LOCKED (store, connect_lock);
 	if (store->server_level >= IMAP_LEVEL_IMAP4REV1)
-		header_spec = "HEADER.FIELDS.NOT (RECEIVED)";
+		header_spec = "HEADER";
 	else
 		header_spec = "0";
 	
@@ -2447,6 +2525,7 @@ imap_update_summary (CamelFolder *folder, int exists,
 			 * have been set by summary_info_new_from_message.
 			 */
 			mi->info.flags |= flags;
+			flags_to_label(folder, mi);
 		}
 		size = GPOINTER_TO_INT (g_datalist_get_data (&data, "RFC822.SIZE"));
 		if (size)
@@ -2600,26 +2679,31 @@ camel_imap_folder_fetch_data (CamelImapFolder *imap_folder, const char *uid,
 	
 	/* EXPUNGE responses have to modify the cache, which means
 	 * they have to grab the cache_lock while holding the
-	 * connect_lock. So we grab the connect_lock now, in case
-	 * we're going to need it below, since we can't grab it
-	 * after the cache_lock.
+	 * connect_lock.
+
+	 * Because getting the service lock may cause MUCH unecessary
+	 * delay when we already have the data locally, we do the
+	 * locking separately.  This could cause a race
+	 * getting the same data from the cache, but that is only
+	 * an inefficiency, and bad luck.
 	 */
-	CAMEL_SERVICE_LOCK (store, connect_lock);
-	
 	CAMEL_IMAP_FOLDER_LOCK (imap_folder, cache_lock);
 	stream = camel_imap_message_cache_get (imap_folder->cache, uid, section_text, ex);
 	if (!stream && (!strcmp (section_text, "HEADER") || !strcmp (section_text, "0"))) {
 		camel_exception_clear (ex);
 		stream = camel_imap_message_cache_get (imap_folder->cache, uid, "", ex);
 	}
+	CAMEL_IMAP_FOLDER_UNLOCK (imap_folder, cache_lock);
 	
-	if (stream || cache_only) {
-		CAMEL_IMAP_FOLDER_UNLOCK (imap_folder, cache_lock);
-		CAMEL_SERVICE_UNLOCK (store, connect_lock);
+	if (stream || cache_only)
 		return stream;
-	}
-	
-	if (camel_disco_store_status (CAMEL_DISCO_STORE (store)) == CAMEL_DISCO_STORE_OFFLINE) {
+
+	camel_exception_clear(ex);
+
+	CAMEL_SERVICE_LOCK (store, connect_lock);
+	CAMEL_IMAP_FOLDER_LOCK (imap_folder, cache_lock);
+
+	if (!camel_imap_store_connected(store, ex)) {
 		camel_exception_set (ex, CAMEL_EXCEPTION_SERVICE_UNAVAILABLE,
 				     _("This message is not currently available"));
 		CAMEL_IMAP_FOLDER_UNLOCK (imap_folder, cache_lock);
