@@ -28,8 +28,6 @@
 #endif
 
 #include <sys/types.h>
-#include <sys/socket.h>
-#include <netinet/in.h>
 #include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
