@@ -114,7 +114,7 @@ impl_dispose (GObject *object)
 /* Initialization.  */
 
 static void
-class_init (ESourceClass *class)
+e_source_class_init (ESourceClass *class)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (class);
 
@@ -134,7 +134,7 @@ class_init (ESourceClass *class)
 }
 
 static void
-init (ESource *source)
+e_source_init (ESource *source)
 {
 	ESourcePrivate *priv;
 
@@ -155,11 +155,11 @@ e_source_get_type (void)
                         sizeof (ESourceClass),
                         (GBaseInitFunc) NULL,
                         (GBaseFinalizeFunc) NULL,
-                        (GClassInitFunc) class_init,
+                        (GClassInitFunc) e_source_class_init,
                         NULL, NULL,
                         sizeof (ESource),
                         0,
-                        (GInstanceInitFunc) init
+                        (GInstanceInitFunc) e_source_init
                 };
 		e_source_type = g_type_register_static (G_TYPE_OBJECT, "ESource", &info, 0);
 	}
@@ -373,7 +373,13 @@ e_source_update_from_xml_node (ESource *source,
 			changed = TRUE;
 		}
 	}
-
+	if (g_hash_table_size (source->priv->properties) && !node->children) {
+		g_hash_table_destroy (source->priv->properties);
+		source->priv->properties = g_hash_table_new_full (g_str_hash, g_str_equal,
+								  g_free, g_free);
+		changed = TRUE;
+	}
+	
 	for (node = node->children; node; node = node->next) {
 		if (!node->name)
 			continue;
@@ -503,6 +509,8 @@ void
 e_source_set_relative_uri (ESource *source,
 			   const char *relative_uri)
 {
+	char *absolute_uri;
+
 	g_return_if_fail (E_IS_SOURCE (source));
 
 	if (source->priv->readonly)
@@ -514,6 +522,13 @@ e_source_set_relative_uri (ESource *source,
 	g_free (source->priv->relative_uri);
 	source->priv->relative_uri = g_strdup (relative_uri);
 
+	/* reset the absolute uri */
+	if (source->priv->absolute_uri && 
+	    (absolute_uri = e_source_build_absolute_uri (source))) {
+		g_free (source->priv->absolute_uri);
+		source->priv->absolute_uri = absolute_uri;
+	}
+
 	g_signal_emit (source, signals[CHANGED], 0);
 }
 
