@@ -30,6 +30,8 @@
 #include <stdlib.h>
 #include <string.h>
 
+static gboolean is_offline ();
+
 void
 add_folder_esource (ExchangeAccount *account, 
 	     	    FolderType folder_type, 
@@ -43,8 +45,10 @@ add_folder_esource (ExchangeAccount *account,
 	GConfClient *client;
 	gboolean is_contacts_folder = TRUE, group_new = FALSE, source_new = FALSE;
 	const char *offline = NULL;
+	char *username, *authtype;
 	int mode;
 	ESourceList *source_list = NULL;
+	gboolean offline_flag;
 
 	client = gconf_client_get_default ();
 
@@ -70,6 +74,8 @@ add_folder_esource (ExchangeAccount *account,
 	}
 
 	exchange_account_is_offline_sync_set (account, &mode);
+	username = exchange_account_get_username (account);
+	authtype = exchange_account_get_authtype (account);
 
         if ((source_group = e_source_list_peek_group_by_name (source_list, 
 					account->account_name)) == NULL) {
@@ -94,6 +100,14 @@ add_folder_esource (ExchangeAccount *account,
 			 */
 			e_source_set_property (source, "offline_sync", "1");
 		}
+		e_source_set_property (source, "username", username);
+		e_source_set_property (source, "auth-domain", "Exchange");
+		if (authtype)
+			e_source_set_property (source, "auth-type", authtype);
+		if (is_contacts_folder)
+			e_source_set_property (source, "auth", "plain/password");
+		else
+			e_source_set_property (source, "auth", "1");
 		e_source_group_add_source (source_group, source, -1);
 		e_source_list_sync (source_list, NULL);
 		group_new = source_new = TRUE;
@@ -112,6 +126,14 @@ add_folder_esource (ExchangeAccount *account,
 			if (mode == OFFLINE_MODE)
 				e_source_set_property (source, "offline_sync", "1");
 
+			e_source_set_property (source, "username", username);
+			e_source_set_property (source, "auth-domain", "Exchange");
+			if (authtype)
+				e_source_set_property (source, "auth-type", authtype);
+			if (is_contacts_folder)
+				e_source_set_property (source, "auth", "plain/password");
+			else
+				e_source_set_property (source, "auth", "1");
 			e_source_group_add_source (source_group, source, -1);
 			source_new = TRUE;
 			e_source_list_sync (source_list, NULL);
@@ -126,10 +148,11 @@ add_folder_esource (ExchangeAccount *account,
 		}
 	}
 
+	offline_flag = is_offline ();
 	if (source && !is_contacts_folder) {
 
 		/* Select the folder created */
-		if (folder_type == EXCHANGE_CALENDAR_FOLDER) {
+		if (folder_type == EXCHANGE_CALENDAR_FOLDER && !offline_flag) {
 			ids = gconf_client_get_list (client,
 					     CONF_KEY_SELECTED_CAL_SOURCES, 
 					     GCONF_VALUE_STRING, NULL);
@@ -141,8 +164,7 @@ add_folder_esource (ExchangeAccount *account,
 			g_slist_foreach (ids, (GFunc) g_free, NULL);
 			g_slist_free (ids);
 		}
-		else if (folder_type == EXCHANGE_TASKS_FOLDER) {
-
+		else if (folder_type == EXCHANGE_TASKS_FOLDER && !offline_flag) {
 			ids = gconf_client_get_list (client, 
 					     CONF_KEY_SELECTED_TASKS_SOURCES, 
 					     GCONF_VALUE_STRING, NULL);
@@ -283,3 +305,20 @@ remove_folder_esource (ExchangeAccount *account,
 	g_object_unref (source_list);
 	g_object_unref (client);
 }
+
+static gboolean 
+is_offline () 
+{
+	GConfClient *client;
+	GConfValue *value;
+	gboolean offline = FALSE;
+
+	client = gconf_client_get_default ();
+	value = gconf_client_get (client,
+					"/apps/evolution/shell/start_offline", NULL);
+	if (value)
+		offline = gconf_value_get_bool (value);
+
+	g_object_unref (client);
+	return offline;
+}
