@@ -28,15 +28,14 @@
 
 #include "exchange-hierarchy-webdav.h"
 #include "exchange-account.h"
-//#include "exchange-constants.h"
 #include "e-folder-exchange.h"
 #include "e2k-context.h"
 #include "e2k-propnames.h"
 #include "e2k-restriction.h"
 #include "e2k-uri.h"
 #include "e2k-utils.h"
-//#include "exchange-config-listener.h"
 #include "exchange-folder-size.h"
+#include "exchange-esource.h"
 
 #include <libedataserverui/e-passwords.h>
 #include "e2k-path.h"
@@ -385,7 +384,6 @@ xfer_folder (ExchangeHierarchy *hier, EFolder *source,
 	const char *folder_type = NULL, *source_folder_name;
 	ExchangeAccountFolderResult ret_code;
 	int offline;
-	gdouble f_size;
 
 	exchange_account_is_offline (hier->account, &offline);
         if (offline != ONLINE_MODE)
@@ -444,8 +442,6 @@ xfer_folder (ExchangeHierarchy *hier, EFolder *source,
 	}
 
 	/* Remove the ESource of the source folder, in case of rename/move */
-#if 0
-SURF :
 	if ((hier->type == EXCHANGE_HIERARCHY_PERSONAL || 
 	     hier->type == EXCHANGE_HIERARCHY_FAVORITES) && remove_source && 
 	     ret_code == EXCHANGE_ACCOUNT_FOLDER_OK) {
@@ -469,7 +465,6 @@ SURF :
 					       physical_uri);
 		}
 	}
-#endif
 	if (physical_uri)
 		g_free (physical_uri);
 	return ret_code;
@@ -493,7 +488,7 @@ add_href (gpointer path, gpointer folder, gpointer hrefs)
 
 /* E2K_PR_EXCHANGE_FOLDER_SIZE also can be used for reading folder size */
 static const char *rescan_props[] = {
-	PR_MESSAGE_SIZE_EXTENDED,
+	E2K_PR_EXCHANGE_FOLDER_SIZE,
 	E2K_PR_HTTPMAIL_UNREAD_COUNT
 };
 static const int n_rescan_props = sizeof (rescan_props) / sizeof (rescan_props[0]);
@@ -548,7 +543,8 @@ rescan (ExchangeHierarchy *hier)
 			e_folder_set_unread_count (folder, unread);
 
 		folder_size = e2k_properties_get_prop (result->props,
-						PR_MESSAGE_SIZE_EXTENDED);
+				E2K_PR_EXCHANGE_FOLDER_SIZE);
+
 		if (folder_size) {
 			folder_name = e_folder_get_name (folder);
 			fsize_d = g_ascii_strtod (folder_size, NULL)/1024;
@@ -642,7 +638,7 @@ exchange_hierarchy_webdav_parse_folder (ExchangeHierarchyWebDAV *hwd,
 	 */
 	permanenturl = e2k_properties_get_prop (result->props,
 						E2K_PR_EXCHANGE_PERMANENTURL);
-	// Check for errors
+	/* Check for errors */
 
 	folder = e_folder_webdav_new (EXCHANGE_HIERARCHY (hwd),
 				      result->href, parent,
@@ -675,7 +671,7 @@ static const char *folder_props[] = {
 	E2K_PR_HTTPMAIL_UNREAD_COUNT,
 	E2K_PR_DAV_DISPLAY_NAME,
 	E2K_PR_EXCHANGE_PERMANENTURL,
-	PR_MESSAGE_SIZE_EXTENDED,
+	E2K_PR_EXCHANGE_FOLDER_SIZE,
 	E2K_PR_DAV_HAS_SUBS
 };
 static const int n_folder_props = sizeof (folder_props) / sizeof (folder_props[0]);
@@ -693,7 +689,7 @@ scan_subtree (ExchangeHierarchy *hier, EFolder *parent, gboolean offline)
 	GPtrArray *folders;
 	int i;
 	gdouble fsize_d;
-	const char *name, *folder_size;
+	const char *name, *folder_size, *deleted_items_uri, *int_uri;
 	gboolean personal = ( EXCHANGE_HIERARCHY (hwd)->type == EXCHANGE_HIERARCHY_PERSONAL );
 
 	if (offline) {
@@ -735,7 +731,7 @@ scan_subtree (ExchangeHierarchy *hier, EFolder *parent, gboolean offline)
 		name = e2k_properties_get_prop (result->props,
 							E2K_PR_DAV_DISPLAY_NAME);
 		folder_size = e2k_properties_get_prop (result->props,
-							PR_MESSAGE_SIZE_EXTENDED);
+				E2K_PR_EXCHANGE_FOLDER_SIZE);
 
 		/* FIXME : Find a better way of doing this */
 		fsize_d = g_ascii_strtod (folder_size, NULL)/1024 ;
@@ -748,9 +744,15 @@ scan_subtree (ExchangeHierarchy *hier, EFolder *parent, gboolean offline)
 	}
 	status = e2k_result_iter_free (iter);
 
+	deleted_items_uri = exchange_account_get_standard_uri (hier->account, "deleteditems");
+
 	while (subtrees) {
 		folder = subtrees->data;
 		subtrees = g_slist_remove (subtrees, folder);
+		/* Dont scan the subtree for deleteditems folder */
+		int_uri = e_folder_exchange_get_internal_uri (folder);
+		if (int_uri && !strcmp (int_uri, deleted_items_uri))
+			continue;
 		scan_subtree (hier, folder, offline);
 	}
 
